package main

import (
	"bytes"
	"embed"
	"image"
	"image/color"
	"image/png"
	"log"
	"math/rand"
	"path/filepath"
	"strings"

	"github.com/fogleman/gg"
	"github.com/golang/freetype/truetype"
	"golang.org/x/image/font"
)

var (
	//go:embed resources/*
	res          embed.FS
	fnt          font.Face
	Images       []image.Image
	CorrectImage int
)

type Item struct {
	img     image.Image
	code    string
	correct bool
}

func InitResources() {
	dir := "resources"
	files, err := res.ReadDir(dir)

	if err != nil {
		log.Fatal(err)
	}

	for i, file := range files {
		name := file.Name()
		p := filepath.Join(dir, name)
		ext := filepath.Ext(name)

		switch ext {
		case ".ttf":
			b, _ := res.ReadFile(p)
			f, _ := truetype.Parse(b)
			fnt = truetype.NewFace(f, &truetype.Options{
				Size: 25,
			})
		case ".png":
			b, _ := res.ReadFile(p)
			img, _, err := image.Decode(bytes.NewReader(b))
			if err != nil {
				log.Fatal("Can't load image: ", err)
			}
			Images = append(Images, img)

			if strings.HasPrefix(name, "correct") {
				CorrectImage = i
				log.Println("Correct one found: ", CorrectImage)
			}
			log.Println(name, "loaded successfully")
		}
	}

}

func GenCaptcha(answer string) ([]byte, string) {
	width, height := 500.0, 500.0
	ctx := gg.NewContext(int(width), int(height))
	// Make gradient
	gradient := gg.NewLinearGradient(0, 0, width, height)
	gradient.AddColorStop(0, color.RGBA{189, 24, 229, 255})
	gradient.AddColorStop(1, color.RGBA{27, 21, 123, 255})

	// Fill background
	ctx.SetFillStyle(gradient)
	ctx.DrawRectangle(0, 0, width, height)
	ctx.Fill()

	// Stripes for background
	ctx.SetRGBA(0, 0, 0, 0.05)
	ctx.SetLineWidth(27)
	for i := 0.0; i < 20; i += 1 {
		ctx.DrawLine(i*70, -100, i*70-500, width)
		ctx.Stroke()
	}

	items := []Item{}
	for i, image := range Images {
		var code string
		if i == CorrectImage {
			code = answer
		} else {
			code = generateRandomDigits(4)
		}
		items = append(items, Item{
			img:     image,
			code:    code,
			correct: i == CorrectImage,
		})
	}
	rand.Shuffle(len(items), func(i, j int) { items[i], items[j] = items[j], items[i] })

	correctAnswer := ""
	minX, maxX := 60, int(width-60)
	xrange := maxX - minX
	step := xrange / (len(items) - 1)
	ctx.SetFontFace(fnt)
	ctx.SetRGBA(1, 1, 1, 0.5)
	for i, img := range items {
		x := minX + i*step
		y := 100 + rand.Intn(int(height)-200)
		if img.correct {
			correctAnswer = img.code
		}
		ctx.DrawImageAnchored(img.img, x, y, 0.5, 0.5)
		offset := 70
		if rand.Float32() < 0.5 {
			offset = -70
		}
		ctx.DrawStringAnchored(img.code, float64(x), float64(y+offset), 0.5, 0.5)
	}

	buff := new(bytes.Buffer)
	err := png.Encode(buff, ctx.Image())
	if err != nil {
		log.Fatal("can't encode png: ", err)
	}
	return buff.Bytes(), correctAnswer
}
