package main

import (
	"context"
	"log"
	"os"
	"os/signal"
	"strings"
	"time"

	_ "github.com/glebarez/go-sqlite"
	"github.com/go-telegram/bot"
	"github.com/go-telegram/bot/models"
	"github.com/joho/godotenv"
)

var BOT_NAME string

func main() {
	log.SetFlags(log.Lshortfile + log.Ltime + log.Ldate)
	InitResources()

	error := godotenv.Load()

	if error != nil {
		log.Println("no .env loaded. relying on existing variables")
	}

	ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt)
	defer cancel()

	if err := InitDb(); err != nil {
		panic(err)
	}
	defer db.Close()

	// db.MustExec(schema)

	opts := []bot.Option{
		bot.WithMiddlewares(logChats, logUsers, checkRegistered),
		bot.WithDefaultHandler(defaultHandler),
	}

	b, error := bot.New(os.Getenv("TG_TOKEN"), opts...)
	if error != nil {
		panic(error)
	}
	me, error := b.GetMe(ctx)
	if error != nil {
		log.Fatalf("Can't get me: %v", error)
	}
	BOT_NAME = me.Username
	log.Println("Using name", BOT_NAME)

	b.RegisterHandler(bot.HandlerTypeMessageText, "/register", bot.MatchTypePrefix, registerChat)
	b.RegisterHandler(bot.HandlerTypeMessageText, "/unregister", bot.MatchTypePrefix, unregisterChat)
	b.RegisterHandler(bot.HandlerTypeMessageText, "/ban", bot.MatchTypePrefix, banUser)
	b.RegisterHandlerMatchFunc(func(update *models.Update) bool {
		return update.Message != nil && len(update.Message.NewChatMembers) > 0
	}, handleNewJoined)
	b.RegisterHandlerMatchFunc(func(update *models.Update) bool {
		return update.Message != nil && update.Message.Chat.Type == "private" && strings.HasPrefix(update.Message.Text, "/start")
	}, handlePrivateStartCaptcha)
	b.RegisterHandlerMatchFunc(func(update *models.Update) bool {
		return update.Message != nil && update.Message.Chat.Type == "private" && !strings.HasPrefix(update.Message.Text, "/start")
	}, handlePrivateCaptcha)

	go timedTask(ctx, b)

	b.Start(ctx)
}

func timedTask(ctx context.Context, b *bot.Bot) {
	timer := time.NewTicker(time.Second * 30)

task:
	for {
		select {
		case <-timer.C:
			deleteOldMessages(ctx, b)
		case <-ctx.Done():
			break task
		}
	}
}

func deleteOldMessages(ctx context.Context, b *bot.Bot) {
	now := time.Now().Unix()
	messages := []MessageToDelete{}
	err := db.Select(&messages, `SELECT * from messagesToDelete WHERE delete_date < $1`, now)
	if err != nil {
		log.Println("delete_old: can't get messages: ", err)
	}
	for _, message := range messages {
		success, err := b.DeleteMessage(ctx, &bot.DeleteMessageParams{ChatID: message.ChatId, MessageID: message.MessageId})
		if !success {
			log.Println("delete_old, message wasn't deleted: ", message.MessageId, " :: ", err)
			db.Exec(`UPDATE messagesToDelete set tries = tries + 1 where id = $1`, message.Id)
			if message.Tries < 3 {
				continue
			}
		}
		db.Exec(`DELETE FROM messagesToDelete WHERE id = $1`, message.Id)
	}
}
